
#define PY_ARRAY_UNIQUE_SYMBOL cweno_ARRAY_API
#include <stdio.h>
#include <Python.h>
#include <numpy/ndarrayobject.h>
PyObject *py_smoothness003 (PyObject * self, PyObject * args);
PyObject *py_weights_left003001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_left003001 (PyObject * self, PyObject * args);
PyObject *py_weights_right003001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_right003001 (PyObject * self, PyObject * args);
PyObject *py_weights_middle003001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_middle003001 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre003002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre003002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre003003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre003003 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto003002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto003002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto003003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto003003 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau003002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau003002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau003003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau003003 (PyObject * self, PyObject * args);
PyObject *py_smoothness004 (PyObject * self, PyObject * args);
PyObject *py_weights_left004001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_left004001 (PyObject * self, PyObject * args);
PyObject *py_weights_right004001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_right004001 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre004002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre004002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre004004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre004004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto004002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto004002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto004004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto004004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau004002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau004002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau004003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau004003 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau004004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau004004 (PyObject * self, PyObject * args);
PyObject *py_smoothness005 (PyObject * self, PyObject * args);
PyObject *py_weights_left005001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_left005001 (PyObject * self, PyObject * args);
PyObject *py_weights_right005001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_right005001 (PyObject * self, PyObject * args);
PyObject *py_weights_middle005001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_middle005001 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre005002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre005002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre005003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre005003 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre005004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre005004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre005005 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre005005 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto005002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto005002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto005003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto005003 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto005004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto005004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto005005 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto005005 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau005002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau005002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau005003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau005003 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau005004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau005004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau005005 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau005005 (PyObject * self, PyObject * args);
PyObject *py_smoothness006 (PyObject * self, PyObject * args);
PyObject *py_weights_left006001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_left006001 (PyObject * self, PyObject * args);
PyObject *py_weights_right006001 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_right006001 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre006002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre006002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre006004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre006004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_legendre006006 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_legendre006006 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto006002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto006002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto006004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto006004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_lobatto006006 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_lobatto006006 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau006002 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau006002 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau006003 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau006003 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau006004 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau006004 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau006005 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau006005 (PyObject * self, PyObject * args);
PyObject *py_weights_gauss_radau006006 (PyObject * self, PyObject * args);
PyObject *py_reconstruct_gauss_radau006006 (PyObject * self, PyObject * args);

static PyMethodDef cwenomethods[] = {
  {"smoothness003", py_smoothness003, METH_VARARGS, ""},
  {"weights_left003001", py_weights_left003001, METH_VARARGS, ""},
  {"reconstruct_left003001", py_reconstruct_left003001, METH_VARARGS, ""},
  {"weights_right003001", py_weights_right003001, METH_VARARGS, ""},
  {"reconstruct_right003001", py_reconstruct_right003001, METH_VARARGS, ""},
  {"weights_middle003001", py_weights_middle003001, METH_VARARGS, ""},
  {"reconstruct_middle003001", py_reconstruct_middle003001, METH_VARARGS, ""},
  {"weights_gauss_legendre003002", py_weights_gauss_legendre003002, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre003002", py_reconstruct_gauss_legendre003002, METH_VARARGS, ""},
  {"weights_gauss_legendre003003", py_weights_gauss_legendre003003, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre003003", py_reconstruct_gauss_legendre003003, METH_VARARGS, ""},
  {"weights_gauss_lobatto003002", py_weights_gauss_lobatto003002, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto003002", py_reconstruct_gauss_lobatto003002, METH_VARARGS, ""},
  {"weights_gauss_lobatto003003", py_weights_gauss_lobatto003003, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto003003", py_reconstruct_gauss_lobatto003003, METH_VARARGS, ""},
  {"weights_gauss_radau003002", py_weights_gauss_radau003002, METH_VARARGS, ""},
  {"reconstruct_gauss_radau003002", py_reconstruct_gauss_radau003002, METH_VARARGS, ""},
  {"weights_gauss_radau003003", py_weights_gauss_radau003003, METH_VARARGS, ""},
  {"reconstruct_gauss_radau003003", py_reconstruct_gauss_radau003003, METH_VARARGS, ""},
  {"smoothness004", py_smoothness004, METH_VARARGS, ""},
  {"weights_left004001", py_weights_left004001, METH_VARARGS, ""},
  {"reconstruct_left004001", py_reconstruct_left004001, METH_VARARGS, ""},
  {"weights_right004001", py_weights_right004001, METH_VARARGS, ""},
  {"reconstruct_right004001", py_reconstruct_right004001, METH_VARARGS, ""},
  {"weights_gauss_legendre004002", py_weights_gauss_legendre004002, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre004002", py_reconstruct_gauss_legendre004002, METH_VARARGS, ""},
  {"weights_gauss_legendre004004", py_weights_gauss_legendre004004, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre004004", py_reconstruct_gauss_legendre004004, METH_VARARGS, ""},
  {"weights_gauss_lobatto004002", py_weights_gauss_lobatto004002, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto004002", py_reconstruct_gauss_lobatto004002, METH_VARARGS, ""},
  {"weights_gauss_lobatto004004", py_weights_gauss_lobatto004004, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto004004", py_reconstruct_gauss_lobatto004004, METH_VARARGS, ""},
  {"weights_gauss_radau004002", py_weights_gauss_radau004002, METH_VARARGS, ""},
  {"reconstruct_gauss_radau004002", py_reconstruct_gauss_radau004002, METH_VARARGS, ""},
  {"weights_gauss_radau004003", py_weights_gauss_radau004003, METH_VARARGS, ""},
  {"reconstruct_gauss_radau004003", py_reconstruct_gauss_radau004003, METH_VARARGS, ""},
  {"weights_gauss_radau004004", py_weights_gauss_radau004004, METH_VARARGS, ""},
  {"reconstruct_gauss_radau004004", py_reconstruct_gauss_radau004004, METH_VARARGS, ""},
  {"smoothness005", py_smoothness005, METH_VARARGS, ""},
  {"weights_left005001", py_weights_left005001, METH_VARARGS, ""},
  {"reconstruct_left005001", py_reconstruct_left005001, METH_VARARGS, ""},
  {"weights_right005001", py_weights_right005001, METH_VARARGS, ""},
  {"reconstruct_right005001", py_reconstruct_right005001, METH_VARARGS, ""},
  {"weights_middle005001", py_weights_middle005001, METH_VARARGS, ""},
  {"reconstruct_middle005001", py_reconstruct_middle005001, METH_VARARGS, ""},
  {"weights_gauss_legendre005002", py_weights_gauss_legendre005002, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre005002", py_reconstruct_gauss_legendre005002, METH_VARARGS, ""},
  {"weights_gauss_legendre005003", py_weights_gauss_legendre005003, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre005003", py_reconstruct_gauss_legendre005003, METH_VARARGS, ""},
  {"weights_gauss_legendre005004", py_weights_gauss_legendre005004, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre005004", py_reconstruct_gauss_legendre005004, METH_VARARGS, ""},
  {"weights_gauss_legendre005005", py_weights_gauss_legendre005005, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre005005", py_reconstruct_gauss_legendre005005, METH_VARARGS, ""},
  {"weights_gauss_lobatto005002", py_weights_gauss_lobatto005002, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto005002", py_reconstruct_gauss_lobatto005002, METH_VARARGS, ""},
  {"weights_gauss_lobatto005003", py_weights_gauss_lobatto005003, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto005003", py_reconstruct_gauss_lobatto005003, METH_VARARGS, ""},
  {"weights_gauss_lobatto005004", py_weights_gauss_lobatto005004, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto005004", py_reconstruct_gauss_lobatto005004, METH_VARARGS, ""},
  {"weights_gauss_lobatto005005", py_weights_gauss_lobatto005005, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto005005", py_reconstruct_gauss_lobatto005005, METH_VARARGS, ""},
  {"weights_gauss_radau005002", py_weights_gauss_radau005002, METH_VARARGS, ""},
  {"reconstruct_gauss_radau005002", py_reconstruct_gauss_radau005002, METH_VARARGS, ""},
  {"weights_gauss_radau005003", py_weights_gauss_radau005003, METH_VARARGS, ""},
  {"reconstruct_gauss_radau005003", py_reconstruct_gauss_radau005003, METH_VARARGS, ""},
  {"weights_gauss_radau005004", py_weights_gauss_radau005004, METH_VARARGS, ""},
  {"reconstruct_gauss_radau005004", py_reconstruct_gauss_radau005004, METH_VARARGS, ""},
  {"weights_gauss_radau005005", py_weights_gauss_radau005005, METH_VARARGS, ""},
  {"reconstruct_gauss_radau005005", py_reconstruct_gauss_radau005005, METH_VARARGS, ""},
  {"smoothness006", py_smoothness006, METH_VARARGS, ""},
  {"weights_left006001", py_weights_left006001, METH_VARARGS, ""},
  {"reconstruct_left006001", py_reconstruct_left006001, METH_VARARGS, ""},
  {"weights_right006001", py_weights_right006001, METH_VARARGS, ""},
  {"reconstruct_right006001", py_reconstruct_right006001, METH_VARARGS, ""},
  {"weights_gauss_legendre006002", py_weights_gauss_legendre006002, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre006002", py_reconstruct_gauss_legendre006002, METH_VARARGS, ""},
  {"weights_gauss_legendre006004", py_weights_gauss_legendre006004, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre006004", py_reconstruct_gauss_legendre006004, METH_VARARGS, ""},
  {"weights_gauss_legendre006006", py_weights_gauss_legendre006006, METH_VARARGS, ""},
  {"reconstruct_gauss_legendre006006", py_reconstruct_gauss_legendre006006, METH_VARARGS, ""},
  {"weights_gauss_lobatto006002", py_weights_gauss_lobatto006002, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto006002", py_reconstruct_gauss_lobatto006002, METH_VARARGS, ""},
  {"weights_gauss_lobatto006004", py_weights_gauss_lobatto006004, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto006004", py_reconstruct_gauss_lobatto006004, METH_VARARGS, ""},
  {"weights_gauss_lobatto006006", py_weights_gauss_lobatto006006, METH_VARARGS, ""},
  {"reconstruct_gauss_lobatto006006", py_reconstruct_gauss_lobatto006006, METH_VARARGS, ""},
  {"weights_gauss_radau006002", py_weights_gauss_radau006002, METH_VARARGS, ""},
  {"reconstruct_gauss_radau006002", py_reconstruct_gauss_radau006002, METH_VARARGS, ""},
  {"weights_gauss_radau006003", py_weights_gauss_radau006003, METH_VARARGS, ""},
  {"reconstruct_gauss_radau006003", py_reconstruct_gauss_radau006003, METH_VARARGS, ""},
  {"weights_gauss_radau006004", py_weights_gauss_radau006004, METH_VARARGS, ""},
  {"reconstruct_gauss_radau006004", py_reconstruct_gauss_radau006004, METH_VARARGS, ""},
  {"weights_gauss_radau006005", py_weights_gauss_radau006005, METH_VARARGS, ""},
  {"reconstruct_gauss_radau006005", py_reconstruct_gauss_radau006005, METH_VARARGS, ""},
  {"weights_gauss_radau006006", py_weights_gauss_radau006006, METH_VARARGS, ""},
  {"reconstruct_gauss_radau006006", py_reconstruct_gauss_radau006006, METH_VARARGS, ""},
  {NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC
initcweno (void)
{
  (void) Py_InitModule ("cweno", cwenomethods);
  import_array ();
}
